import type { OIDCError } from '../types';

/**
 * OIDC错误处理工厂类
 * 用于创建标准化的OIDC错误响应
 */
export class OIDCErrorFactory {
    /**
     * 创建授权错误（带可选的重定向URI和state）
     */
    static createAuthError(error: string, description: string, state?: string) {
        return {
            success: false as const,
            error: { error, error_description: description, state },
            redirectUri: undefined as string | undefined,
        };
    }

    /**
     * 创建令牌错误
     */
    static createTokenError(error: string, description: string) {
        return {
            success: false as const,
            error: { error, error_description: description },
        };
    }

    /**
     * 创建简单错误（用于一般API响应）
     */
    static createSimpleError(error: string, description: string): OIDCError {
        return { error, error_description: description };
    }

    /**
     * 服务器错误
     */
    static serverError(state?: string) {
        return this.createAuthError('server_error', 'Internal server error', state);
    }

    /**
     * 无效令牌错误
     */
    static invalidToken(description = 'Invalid token') {
        return this.createSimpleError('invalid_token', description);
    }

    /**
     * 无效请求错误
     */
    static invalidRequest(description: string) {
        return this.createSimpleError('invalid_request', description);
    }

    /**
     * 无效客户端错误
     */
    static invalidClient(description: string) {
        return this.createTokenError('invalid_client', description);
    }

    /**
     * 无效授权错误
     */
    static invalidGrant(description: string) {
        return this.createTokenError('invalid_grant', description);
    }

    /**
     * 不支持的授权类型错误
     */
    static unsupportedGrantType(description = 'Grant type not supported') {
        return this.createTokenError('unsupported_grant_type', description);
    }

    /**
     * 无效作用域错误
     */
    static invalidScope(description: string) {
        return this.createTokenError('invalid_scope', description);
    }

    /**
     * 需要登录错误
     */
    static loginRequired(description = 'User authentication is required', state?: string) {
        return this.createAuthError('login_required', description, state);
    }

    /**
     * PKCE相关错误
     */
    static pkceError(description: string, state?: string) {
        return this.createAuthError('invalid_request', description, state);
    }

    /**
     * 构建错误响应的URL参数
     */
    static buildErrorResponse(error: OIDCError): URLSearchParams {
        const params = new URLSearchParams();
        Object.entries(error).forEach(([key, value]) => {
            if (value != null) params.set(key, String(value));
        });
        return params;
    }
} 