import { SignJWT, jwtVerify, importJWK, exportJWK, generateKeyPair } from 'jose';
import type { OIDCUser } from '../types';

export interface JWTPayload {
    iss: string;
    sub: string;
    aud: string | string[];
    exp: number;
    iat: number;
    auth_time?: number;
    nonce?: string;
    acr?: string;
    amr?: string[];
    azp?: string;
    [key: string]: any;
}

export interface AccessTokenPayload extends JWTPayload {
    scope: string;
    client_id: string;
    token_type: 'access_token';
}

export interface IDTokenPayload extends JWTPayload {
    nonce?: string;
    auth_time: number;
    [key: string]: any;
}

/**
 * JWT工具类
 */
export class JWTUtils {
    private signingKey: string;
    private algorithm: 'HS256' | 'RS256' | 'ES256' = 'HS256';

    constructor(signingKey: string, algorithm?: 'HS256' | 'RS256' | 'ES256') {
        this.signingKey = signingKey;
        this.algorithm = algorithm || 'HS256';
    }

    /**
     * 生成访问令牌
     */
    async generateAccessToken(payload: {
        issuer: string;
        subject: string;
        audience: string;
        clientId: string;
        scope: string;
        expiresIn: number;
    }): Promise<string> {
        const now = Math.floor(Date.now() / 1000);

        const jwtPayload: AccessTokenPayload = {
            iss: payload.issuer,
            sub: payload.subject,
            aud: payload.audience,
            exp: now + payload.expiresIn,
            iat: now,
            scope: payload.scope,
            client_id: payload.clientId,
            token_type: 'access_token',
        };

        return await new SignJWT(jwtPayload)
            .setProtectedHeader({ alg: this.algorithm })
            .sign(new TextEncoder().encode(this.signingKey));
    }

    /**
     * 生成ID令牌
     */
    async generateIDToken(payload: {
        issuer: string;
        subject: string;
        audience: string;
        user: OIDCUser;
        authTime: number;
        nonce?: string;
        expiresIn: number;
        requestedClaims?: string[];
    }): Promise<string> {
        const now = Math.floor(Date.now() / 1000);

        const jwtPayload: IDTokenPayload = {
            iss: payload.issuer,
            sub: payload.subject,
            aud: payload.audience,
            exp: now + payload.expiresIn,
            iat: now,
            auth_time: payload.authTime,
        };

        // 添加nonce（如果提供）
        if (payload.nonce) {
            jwtPayload.nonce = payload.nonce;
        }

        // 添加用户声明
        this.addUserClaims(jwtPayload, payload.user, payload.requestedClaims);

        return await new SignJWT(jwtPayload)
            .setProtectedHeader({ alg: this.algorithm })
            .sign(new TextEncoder().encode(this.signingKey));
    }

    /**
     * 验证JWT令牌
     */
    async verifyToken(token: string): Promise<JWTPayload> {
        try {
            const { payload } = await jwtVerify(
                token,
                new TextEncoder().encode(this.signingKey)
            );
            return payload as JWTPayload;
        } catch (error) {
            throw new Error(`Invalid token: ${error instanceof Error ? error.message : 'Unknown error'}`);
        }
    }

    /**
     * 解码JWT令牌（不验证签名）
     */
    decodeToken(token: string): JWTPayload {
        try {
            const parts = token.split('.');
            if (parts.length !== 3) {
                throw new Error('Invalid token format');
            }

            const payloadPart = parts[1];
            if (!payloadPart) {
                throw new Error('Invalid token payload');
            }

            const payload = JSON.parse(
                Buffer.from(payloadPart, 'base64url').toString('utf-8')
            );
            return payload;
        } catch (error) {
            throw new Error(`Failed to decode token: ${error instanceof Error ? error.message : 'Unknown error'}`);
        }
    }

    /**
     * 检查令牌是否过期
     */
    isTokenExpired(token: string): boolean {
        try {
            const payload = this.decodeToken(token);
            const now = Math.floor(Date.now() / 1000);
            return payload.exp < now;
        } catch {
            return true;
        }
    }

    /**
     * 获取令牌剩余有效时间（秒）
     */
    getTokenTTL(token: string): number {
        try {
            const payload = this.decodeToken(token);
            const now = Math.floor(Date.now() / 1000);
            return Math.max(0, payload.exp - now);
        } catch {
            return 0;
        }
    }

    /**
     * 添加用户声明到JWT载荷
     */
    private addUserClaims(
        payload: IDTokenPayload,
        user: OIDCUser,
        requestedClaims?: string[]
    ): void {
        // 标准声明映射
        const standardClaims = {
            name: user.name,
            given_name: user.given_name,
            family_name: user.family_name,
            preferred_username: user.username,
            profile: user.profile,
            picture: user.picture,
            website: user.website,
            email: user.email,
            email_verified: user.email_verified,
            gender: user.gender,
            birthdate: user.birthdate,
            zoneinfo: user.zoneinfo,
            locale: user.locale,
            phone_number: user.phone_number,
            phone_number_verified: user.phone_number_verified,
            address: user.address,
            updated_at: user.updated_at,
        };

        // 如果指定了请求的声明，只添加这些声明
        if (requestedClaims && requestedClaims.length > 0) {
            for (const claim of requestedClaims) {
                if (claim in standardClaims && standardClaims[claim as keyof typeof standardClaims] !== undefined) {
                    payload[claim] = standardClaims[claim as keyof typeof standardClaims];
                } else if (claim in user && user[claim] !== undefined) {
                    payload[claim] = user[claim];
                }
            }
        } else {
            // 添加所有可用的标准声明
            for (const [claim, value] of Object.entries(standardClaims)) {
                if (value !== undefined) {
                    payload[claim] = value;
                }
            }
        }
    }

    /**
     * 生成JWKS（JSON Web Key Set）
     */
    async generateJWKS(): Promise<{ keys: any[] }> {
        // 对于HMAC算法，我们不暴露密钥
        // 这里返回空的JWKS，实际应用中可能需要使用RSA或ECDSA
        return { keys: [] };
    }

    /**
     * 生成RSA密钥对（用于生产环境）
     */
    static async generateRSAKeyPair(): Promise<{
        privateKey: string;
        publicKey: string;
        jwk: any;
    }> {
        const { privateKey, publicKey } = await generateKeyPair('RS256');
        const jwk = await exportJWK(publicKey);

        return {
            privateKey: JSON.stringify(await exportJWK(privateKey)),
            publicKey: JSON.stringify(jwk),
            jwk: {
                ...jwk,
                alg: 'RS256',
                use: 'sig',
                kid: 'default',
            },
        };
    }
} 