import { prisma } from '@repo/db';
import type { Resource } from '@repo/db';
import { StorageType } from '../types';

export async function getResourceByFileId(fileId: string): Promise<{ status: string; resource?: Resource }> {
	const resource = await prisma.resource.findFirst({
		where: { fileId },
	});

	if (!resource) {
		return { status: 'pending' };
	}

	return {
		status: resource.status || 'unknown',
		resource,
	};
}

export async function getAllResources(): Promise<Resource[]> {
	return prisma.resource.findMany({
		orderBy: { createdAt: 'desc' },
	});
}

export async function getResourcesByStorageType(storageType: StorageType): Promise<Resource[]> {
	return prisma.resource.findMany({
		where: {
			storageType: storageType,
		},
		orderBy: { createdAt: 'desc' },
	});
}

export async function getResourcesByStatus(status: string): Promise<Resource[]> {
	return prisma.resource.findMany({
		where: { status },
		orderBy: { createdAt: 'desc' },
	});
}

export async function getUploadingResources(): Promise<Resource[]> {
	return prisma.resource.findMany({
		where: {
			status: 'UPLOADING',
		},
		orderBy: { createdAt: 'desc' },
	});
}

export async function getResourceStats(): Promise<{
	total: number;
	byStatus: Record<string, number>;
	byStorageType: Record<string, number>;
}> {
	const [total, statusStats, storageStats] = await Promise.all([
		prisma.resource.count(),
		prisma.resource.groupBy({
			by: ['status'],
			_count: true,
		}),
		prisma.resource.groupBy({
			by: ['storageType'],
			_count: true,
		}),
	]);

	const byStatus = statusStats.reduce(
		(acc, item) => {
			acc[item.status || 'unknown'] = item._count || 0;
			return acc;
		},
		{} as Record<string, number>,
	);

	const byStorageType = storageStats.reduce(
		(acc, item) => {
			const key = (item.storageType as string) || 'unknown';
			acc[key] = item._count;
			return acc;
		},
		{} as Record<string, number>,
	);

	return {
		total,
		byStatus,
		byStorageType,
	};
}

export async function deleteResource(id: string): Promise<Resource> {
	return prisma.resource.delete({
		where: { id },
	});
}

export async function updateResource(id: string, data: any): Promise<Resource> {
	return prisma.resource.update({
		where: { id },
		data,
	});
}

export async function migrateResourcesStorageType(
	fromStorageType: StorageType,
	toStorageType: StorageType,
): Promise<{ count: number }> {
	const result = await prisma.resource.updateMany({
		where: {
			storageType: fromStorageType,
		},
		data: {
			storageType: toStorageType,
		},
	});

	return { count: result.count };
}

export async function createResource(data: {
	fileId: string;
	filename: string;
	size: number;
	mimeType?: string | null;
	storageType: StorageType;
	status?: string;
	hash?: string;
}): Promise<Resource> {
	return prisma.resource.create({
		data: {
			fileId: data.fileId,
			title: data.filename,
			type: data.mimeType,
			storageType: data.storageType,
			status: data.status || 'UPLOADING',
			meta: {
				size: data.size,
				hash: data.hash,
			},
		},
	});
}

export async function updateResourceStatus(fileId: string, status: string, additionalData?: any): Promise<Resource> {
	return prisma.resource.update({
		where: { fileId },
		data: {
			status,
			...additionalData,
		},
	});
}
